/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include "synchronize.h"

using namespace std;

void REMOVE(const char *name)
{
    if ((sem_unlink(name) == -1) && (errno != ENOENT))
    {
        printf("*** Semaphore '%s' unlink error!\n", name);
        exit(errno);
    }
}

sem_t *CREATE(Semaphore *semaphore, char *name, int value)
{
    semaphore->name = name;
    semaphore->sem  = sem_open(name, O_CREAT, 0666, value);

    if (semaphore->sem == SEM_FAILED)
    {
        printf("*** Semaphore '%s' create error!\n", name);
        exit(errno);
    }

    return semaphore->sem;
}

sem_t *OPEN(Semaphore *semaphore, char *name)
{
    semaphore->name = name;
    semaphore->sem  = sem_open(semaphore->name, 0);

    if (semaphore->sem == SEM_FAILED)
    {
        printf("*** Semaphore '%s' open error!\n", semaphore->name);
        exit(-23);
    }

    return semaphore->sem;
}

void WAIT(Semaphore *semaphore)
{
    if (sem_wait(semaphore->sem) == -1)
    {
        printf("*** Semaphore '%s' wait error!\n", semaphore->name);
        exit(-24);
    }
}

void SIGNAL(Semaphore *semaphore)
{
    if (sem_post(semaphore->sem) == -1)
    {
        printf("*** Semaphore '%s' signal error!\n", semaphore->name);
        exit(-25);
    }
}
